
var CharacterStyle = {Current:0, Replay:1, Enemy:2}

CharacterObj = function ( Object, Type) {
	
	this.Object = Object;
	this.StartPosition = this.Object.position.clone();
	this.Type = Type;
	this.BaseGravity = 13.0;	
	this.RandTimeDecal = Math.random() * 10.0;
	this.IsSpawner = false;
	
	this.Gravity = -this.BaseGravity;
	this.CanJump = false;	
	this.PreviousJumpTime = -1;
	this.PreviousJumpAsk = -1;
	this.PreviousShoot = -1;
	this.Life = 100;
	this.Protected = -1;
	this.Dead = false;
	this.DeadAnim = 0;	
	this.LocalTime = 0.0;
	this.AnimLeft = false;
	this.AnimRight = false;
	this.AnimJump = false;
	
	this.ResetGravity = -this.BaseGravity;
	this.ResetCanJump = false;	
	this.ResetPreviousJumpTime = -1;
	this.ResetPreviousJumpAsk = -1;
	this.ResetPreviousShoot = -1;
	this.ResetLife = 100;
	this.ResetProtected = -1;
	this.ResetDead = false;
	this.ResetDeadAnim = 0;	
	this.ResetLocalTime = 0.0;
	this.ResetAnimLeft = false;
	this.ResetAnimRight = false;
	this.ResetAnimJump = false;
};

CharacterObj.prototype = {

	constructor: CharacterObj,
		
	RenderUpdate: function( Time, Delta )
	{	
		var CurrentDelta = Delta;
		if(this.Dead)
			CurrentDelta = 0.0
		if(!this.AnimLeft && !this.AnimRight &&  !this.AnimJump) {
			CurrentDelta *= 0.2;
		}
		this.LocalTime += CurrentDelta;
		
		this.Object.material.uniforms.time.value = (this.LocalTime + this.RandTimeDecal)*3.0;
		
		if(!this.AnimLeft && this.AnimRight) {
			this.Object.scale.x = -1.0;
		} else if(this.AnimLeft && !this.AnimRight) {
			this.Object.scale.x = 1.0;
		}
		
		var LifeBar = this.Life/100.0;
		var Intensity = 0.6*LifeBar;
		if((this.Protected>0)) {
			Intensity *= 0.8;
		}
		if(this.Type == CharacterStyle.Current) {
			this.Object.material.uniforms.col.value.x = Intensity;
			this.Object.material.uniforms.col.value.y = Intensity;
		} else if(this.Type == CharacterStyle.Replay) {
			this.Object.material.uniforms.col.value.x = Intensity;
			this.Object.material.uniforms.col.value.z = Intensity;
		} else if(this.Type == CharacterStyle.Enemy){
			this.Object.material.uniforms.col.value.y = Intensity;
			this.Object.material.uniforms.col.value.z = Intensity;
		}
		if(this.IsSpawner) {
			this.Object.material.uniforms.col.value.x = 0.0;
			this.Object.material.uniforms.col.value.y = Intensity;
			this.Object.material.uniforms.col.value.z = 0.0;
		}
		if(this.Dead) {
			var DeathOpa = this.DeadAnim/16.0;
			this.Object.material.uniforms.col.value.w = DeathOpa;
		} else {
			this.Object.material.uniforms.col.value.w = 1.0;
		}
	},
	
	Reset: function()
	{	
		this.Object.position.copy(this.StartPosition)
		
		this.Gravity = this.ResetGravity;
		this.CanJump = this.ResetCanJump;	
		this.PreviousJumpTime = this.ResetPreviousJumpTime;
		this.PreviousJumpAsk = this.ResetPreviousJumpAsk;
		this.PreviousShoot = this.ResetPreviousShoot;
		this.Life = this.ResetLife;
		this.Protected = this.ResetProtected;
		this.Dead = this.ResetDead;
		this.DeadAnim = this.ResetDeadAnim;	
		this.LocalTime = this.ResetLocalTime;
		this.AnimLeft = this.ResetAnimLeft;
		this.AnimRight = this.ResetAnimRight;
		this.AnimJump = this.ResetAnimJump;
	},
	
	KeepState: function()
	{	
		this.Object.position.copy(this.StartPosition)
		
		this.ResetGravity = this.Gravity;
		this.ResetCanJump = this.CanJump;	
		this.ResetPreviousJumpTime = this.PreviousJumpTime;
		this.ResetPreviousJumpAsk = this.PreviousJumpAsk;
		this.ResetPreviousShoot = this.PreviousShoot;
		this.ResetLife = this.Life;
		this.ResetProtected = this.Protected;
		this.ResetDead = this.Dead;
		this.ResetDeadAnim = this.DeadAnim;	
		this.ResetLocalTime = this.LocalTime;
		this.ResetAnimLeft = this.AnimLeft;
		this.ResetAnimRight = this.AnimRight;
		this.ResetAnimJump = this.AnimJump;
	},
	
	ParticleCol: function()
	{	
		if(this.Dead || this.Protected>0) {
			return;
		}
		var TestDist = 0.5;
		var DoubleTestDist = TestDist*TestDist;
		for(var i=0; i<Particles.length; ++i) {
			if(Particles[i].Killing) {
				var DX = this.Object.position.x - Particles[i].TestLocation.x;
				var DY = this.Object.position.y - Particles[i].TestLocation.y;
				var DoubleDist = DX*DX + DY*DY;
				if(DoubleDist<DoubleTestDist) {
					this.Life -= 50;
					this.Protected = 12;
					break;
				}
			}
			/*
			if(i<DebugPlanes.length) {
				DebugPlanes[i].position.copy(Particles[i].TestLocation);
				DebugPlanes[i].position.z = 0.0;
			}*/
		}
	},
	
	EnemyCol: function()
	{	
		if(this.Dead || this.Protected>0) {
			return;
		}
		var TestDistX = 0.3;
		var TestDistY = 0.5;
		for(var i=0; i<AllEnemies.length; ++i) {
			if(!AllEnemies[i].Dead) {
				var DX = Math.abs(this.Object.position.x - AllEnemies[i].Object.position.x);
				var DY = Math.abs(this.Object.position.y - AllEnemies[i].Object.position.y);
				if(DX<TestDistX && DY<TestDistY) {
					this.Life -= 10;
					this.Protected = 12;
					break;
				}
			}
		}
	},
		
	KeyMove: function( Sample )
	{
	
		if(this.Life<0) {
			this.Dead = true;
			this.DeadAnim = 16;
			this.Life = 0;
		}
		
		if(this.DeadAnim>0) {
			--this.DeadAnim;
		}
	
		var NewPosX = this.Object.position.x;
		var NewPosY = this.Object.position.y;
		
		this.AnimLeft = false;
		this.AnimRight = false;
		this.AnimJump = false;
		
		if(!this.Dead) {
			if(Sample.KeyLeft) {
				NewPosX -= 5.0 * FixedDeltaTime;
				this.AnimLeft = true;
	
			}
			if(Sample.KeyRight) {
				NewPosX += 5.0 * FixedDeltaTime;
				this.AnimRight = true;
			}
			
			if(Sample.KeyUp || this.PreviousJumpAsk>0) { 
				if(Sample.KeyUp) {
					this.PreviousJumpAsk = 4;
				}
				if(this.CanJump || this.PreviousJumpTime == 0) {
					this.Gravity = this.BaseGravity;
					this.AnimJump = true;
					this.PreviousJumpTime = -1;
				}
				if(this.CanJump) {
					this.PreviousJumpTime = 9;
				}			
			}
		}
		if(this.PreviousJumpTime>0) {
			--this.PreviousJumpTime;
		}
		if(this.PreviousJumpAsk>0) {
			--this.PreviousJumpAsk;
		}
		
		if(this.Gravity > -this.BaseGravity) {
			this.Gravity -= this.BaseGravity * FixedDeltaTime * 4.0;
		} else {
			this.Gravity = -this.BaseGravity;
		}
		
		var FallPosY = NewPosY + this.Gravity * FixedDeltaTime;
		
		var DecalTestX = -0.5/CurrentLevel.Scale;
		var DecalTestY = -0.5/CurrentLevel.Scale;
	
		var SmoothCol = CurrentLevel.GetSmoothColl(NewPosX+DecalTestX, NewPosY+DecalTestY, true);
		var SmoothFallCol = CurrentLevel.GetSmoothColl(NewPosX+DecalTestX, FallPosY+DecalTestY, false);
			
		if(SmoothCol || SmoothFallCol) {
			if(SmoothFallCol) this.CanJump = true;
			this.Gravity = 0.0;
		} else {
			this.CanJump = false;
		}
		
		if(SmoothCol) {
			if(SmoothCol != 2) {
				this.Object.position.x = NewPosX;
				this.Object.position.y = NewPosY + 7.0 * FixedDeltaTime;
			}
		} else {
			this.Object.position.x = NewPosX;
			this.Object.position.y = SmoothFallCol ? NewPosY : FallPosY;
		}
		
		if(!this.Dead) {
			if(Sample.MouseDown && this.PreviousShoot <= 0) {
				var ParticleCenter = this.Object.position.clone();
				ParticleCenter.y += 0.33;
				var FireDirection = new THREE.Vector2();
				FireDirection.x = Sample.MouseLocation.x-ParticleCenter.x;
				FireDirection.y = Sample.MouseLocation.y-ParticleCenter.y;
				FireDirection.normalize();		
				var NewParticle = new PartSystem( FireDirection );
				NewParticle.Object.position.copy(ParticleCenter);
				Particles.push( NewParticle );
				
				this.PreviousShoot = 8;
			}
		}
		
		if(this.PreviousShoot>0) {
			--this.PreviousShoot;
		}
		
		if(this.Protected>0) {
			--this.Protected;
		}
	},

};