
Level = function ( image ) {

	this.width = image.width;
	this.height = image.height;
	
	this.Scale = 2.0;
		
	this.Collision = [];
	this.SpawnsX = [];
	this.SpawnsY = [];
	
	var canvas = document.createElement( 'canvas' );
	canvas.width = image.width;
	canvas.height = image.height;

	var context = canvas.getContext( '2d' );
	context.drawImage( image, 0, 0 );

	var data = context.getImageData( 0, 0, image.width, image.height ).data;

	for ( var y = 0; y < image.height; y ++ ) {
		for ( var x = 0; x < image.width; x ++ ) {
			var Index = (y * image.width + x) * 4;
			this.Collision.push(data[Index+0]); // Red
			if(data[Index+1]>0) { // Green
				this.SpawnsX.push(x / this.Scale); 
				this.SpawnsY.push((this.height-y-1) / this.Scale); 
			}
		}
	}	
};


Level.prototype = {

	constructor: Level,
	
	GetIdx : function( vX, vY)
	{
		var vScaleX = Math.floor(vX * this.Scale);
		var vScaleY = Math.floor(vY * this.Scale);
		
		if(vScaleX < 0 || vScaleX >= this.width || vScaleY < 0 || vScaleY >= this.height)
		{
			return -1;
		}
		
		return vScaleX + (this.height-vScaleY-1) * this.width;
	},
	
	GetIdxOffset : function( vX, vY, offX, offY)
	{
		var vScaleX = Math.floor(vX * this.Scale + offX);
		var vScaleY = Math.floor(vY * this.Scale + offY);
		
		if(vScaleX < 0 || vScaleX >= this.width || vScaleY < 0 || vScaleY >= this.height)
		{
			return -1;
		}
		
		return vScaleX + (this.height-vScaleY-1) * this.width;
	},
	
	IsDirectColl: function( idx )
	{		
		return (this.Collision[idx]>0);
	},
	
	GetColl: function( vX, vY )
	{		
		var idx = this.GetIdx(vX, vY);
		if(idx<0) return true;
		
		return this.IsDirectColl(idx);
	},
	
	GetSmoothColl: function( vX, vY, drawDebug )
	{		
		var idx1 = this.GetIdx(vX, vY);
		var idx2 = this.GetIdxOffset(vX, vY, 1, 0);
		var idx3 = this.GetIdxOffset(vX, vY, 0, 1);
		var idx4 = this.GetIdxOffset(vX, vY, 1, 1);
		
        var col1 = this.IsDirectColl(idx1);
        var col2 = this.IsDirectColl(idx2);
        var col3 = this.IsDirectColl(idx3);
        var col4 = this.IsDirectColl(idx4);
		/*
		if(drawDebug) {
			DebugPlanes[0].position.x = vX;
			DebugPlanes[0].position.y = vY;
			DebugPlanes[0].material.color.setRGB(col1,0,0);
			
			DebugPlanes[1].position.x = vX + 1.0/this.Scale;
			DebugPlanes[1].position.y = vY;
			DebugPlanes[1].material.color.setRGB(col2,0,0);
			
			DebugPlanes[2].position.x = vX;
			DebugPlanes[2].position.y = vY + 1.0/this.Scale;
			DebugPlanes[2].material.color.setRGB(col3,0,0);
			
			DebugPlanes[3].position.x = vX + 1.0/this.Scale;
			DebugPlanes[3].position.y = vY + 1.0/this.Scale;
			DebugPlanes[3].material.color.setRGB(col4,0,0);
		}
		*/
		if(!col1 && !col2) return 0;
		if(col1 && col3 && !(col2 && !col4)) return 2;
		if(col2 && col4 && !(col1 && !col3)) return 2;
		
		var v1 = col1 ? 1.0 : 0.0;
		var v2 = col2 ? 1.0 : 0.0;
		var vScaleY = Math.floor(vY * this.Scale);
		var modulo = (vX * this.Scale) % 1.0;
		var smoothCol = v1 * (1.0-modulo) + v2 * modulo;
		
		//if(vX < vScaleY + smoothCol * this.Scale) return true;
		
		return ((vScaleY + smoothCol) / this.Scale > vY) ? 1 : 0;
		
		/*
		if(!col1 && !col2) return 0.0;
		//if(col1 && col2 && (col3 != col4)) return 0.0;
		if(col1 && col3 && !(col2 && !col4)) return 1000.0;
		if(col2 && col4 && !(col1 && !col3)) return 1000.0;
		
		var v1 = col1 ? 1.0 : 0.0;
		var v2 = col2 ? 1.0 : 0.0;
		var vScaleY = Math.floor(vY * this.Scale);
		var modulo = (vX * this.Scale) % 1.0;
		var smoothCol = v1 * (1.0-modulo) + v2 * modulo;
		
		//if(vX < vScaleY + smoothCol * this.Scale) return true;
		
		return ((vScaleY + smoothCol) / this.Scale - vY);
		*/
	},
		
};